/************************************************************************
 *
 *  Module:     NETUSBtest.c 
 *  Long name:    
 *
 *  Runtime Env.: Linux
 *  Author(s):    gde (net gmbh)
 *  Company:      Net GmbH, Finning    http://www.net-gmbh.com
 *
 *  ROI MIMR TEST PROGRAM (Multiple Integration Multiple Roi)
 *
 *  for ICube models: 4133bu/cu, 4203bu/cu
 *  for Api version 1.29 or above 
 *
 ************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>

#include "NETUSBCAM_API.h"
#include "ICubeDefines.h"
 
//---------------------------------------------------------------------------
// PROTOTYPES

void SignalHandler(int i);
void SetRoiID(int nRoiID);
void SetExposure(int nRoiID,float nVal);
void SetGain(int nRoiID,int nVal);
void CheckResolutionDelta(int nRes);
void SaveRaw(unsigned char *buffer, unsigned int buffersize, const char* cName);

//---------------------------------------------------------------------------
// DEFINES

#define TRIGGERMODE 		// camera runs in trigger mode
#define TRIGGER_TIMEOUT 1000 

#define MAXROI_NUM	4 	// maximal number of rois
#define X_SIZE 		1280	// maximal size
#define Y_SIZE 		1024	// maximal size
#define ROI_DELTA	4


//#define DEFECT_CORRECTION	// this sets the defect pixel corretion active
//#define SAVE_BMP		// save a bmp
//#define SAVE_RAW  		// save a raw
//#define ROI_CYCLE		// define an aquisition cycle of each roi

//---------------------------------------------------------------------------
// GLOBALS
 
int CamIndex = 0; 
int nRoiAnz = 2;		// number of rois

bool bImageSwTrig=false;	// callback flag
bool bExit=false;

unsigned int nGoodCnt[MAXROI_NUM]={0}; 
unsigned int nBadCnt[MAXROI_NUM]={0}; 

unsigned int nRoisCnt = 0; 
unsigned int nRoisExpected = 0; 
unsigned int nAllRoisCnt = 0; 

int nXRes[MAXROI_NUM]={X_SIZE};	// width of rois
int nYRes[MAXROI_NUM]={Y_SIZE}; // height of rois


//---------------------------------------------------------------------------
// Callback
//
int GetImage(void *buffer, unsigned int buffersize, void *context)
{
  long nRoiIndex = reinterpret_cast<long>(context);

  int nInd = nRoiIndex-1;

  if(buffersize==0){		// badframe arrived (this happens here, when (REG_CALLBACK_BR_FRAMES==1)
	nBadCnt[nInd]++;
  }
  else				// good frame arrived
  { 
	nGoodCnt[nInd]++; 
	
	char buf[24];
	#ifdef SAVE_RAW
	sprintf(buf,"Nr%dROI%d_%dx%d.raw",nGoodCnt[nInd],(int)nRoiIndex,nXRes[nInd],nYRes[nInd]);	
	SaveRaw((unsigned char*)buffer,buffersize,buf);
	#endif
	#ifdef SAVE_BMP
	sprintf(buf,"Nr%dROI%d_%dx%d.bmp",nGoodCnt[nInd],(int)nRoiIndex,nXRes[nInd],nYRes[nInd]);	
	NETUSBCAM_SaveToFile(CamIndex,buf);
	#endif
  }
  
  nRoisCnt++;
  if(nRoisCnt==nRoisExpected)	// all rois are arrived (good + bad frames)
  {
	nRoisCnt=0;
	nAllRoisCnt++;
	bImageSwTrig = true;
  }
  printf("Got Roi_%d;  GoodFr: %d ,BadFr: %d , Size %d  \n",(int)nRoiIndex,nGoodCnt[nInd],nBadCnt[nInd],buffersize);  
}
//---------------------------------------------------------------------------
//
int main()
{ 
	long * context; 
	int i = 0 , result = 0;
	
	int nCnt=0;
	long nRoiID = ROI_ID_1_1; 

	unsigned int nCycleCnt[MAXROI_NUM]={1,0,0,0}; // aquisition cycles of rois (0 means 1 roi)	 

	float fExp[MAXROI_NUM]={10,10,10,10};  	// exposure time of roi [msec]
	int nGain[MAXROI_NUM]={1,1,1,1};	// gain time of roi
	
	int nXPos[MAXROI_NUM]={0,0,0,0}; 	// starting point of roi
	int nYPos[MAXROI_NUM]={0,0,0,0};	// starting point of roi

  	CheckResolutionDelta(nXRes[0]=512); 
	CheckResolutionDelta(nYRes[0]=200);
	CheckResolutionDelta(nXRes[1]=512);
	CheckResolutionDelta(nYRes[1]=200);
	CheckResolutionDelta(nXRes[2]=512);
	CheckResolutionDelta(nYRes[2]=200);
	CheckResolutionDelta(nXRes[3]=512);
	CheckResolutionDelta(nYRes[3]=200);

	printf("----Multiple Integration Multiple Roi----\n\n");

	result = NETUSBCAM_Init();
	if(result==0){
		printf("No device\n");
		return 0; }	

  	signal(SIGINT, SignalHandler);		// register signal handler for Ctrl+C

	result = NETUSBCAM_Open(CamIndex);	// open one camera (camera 0)
	if(result!=0){
		printf("Error: Open; Result = %d\n", result);
		return 0; }

	char cCamName[20];			// check camera model
	result=NETUSBCAM_GetName(CamIndex,cCamName,20);
	if(result!=0){
		printf("Error: GetName; Result = %d\n", result);
		return 0; }

	printf("Model name: %s \n\n",cCamName);
	if(!(	(strcmp(cCamName,"NET 4133 BU")==0) || (strcmp(cCamName,"NET 4133 CU")==0)||
		(strcmp(cCamName,"NET3 4133 BU")==0) || (strcmp(cCamName,"NET3 4133 CU")==0)||
		(strcmp(cCamName,"NET 4133 IR")==0) || (strcmp(cCamName,"NET3 4133 IR")==0)||
		(strcmp(cCamName,"NET 4203 BU")==0) || (strcmp(cCamName,"NET 4203 CU")==0)||
		(strcmp(cCamName,"NET3 4203 BU")==0) || (strcmp(cCamName,"NET3 4203 CU")==0)
		)){
		printf("Error: this example only works with NET4133bu/cu cameras\n");
		NETUSBCAM_Close(CamIndex);
		return 0; }
		
 	// if too many bad frames arriving, and reducing REG_PLL does not help,
	// slow down sensor timing a little bit (increase value)
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_SENSOR_TIMING,0);
	if(result!=0){
		printf("Error: REG_SENSOR_TIMING; Result = %d\n", result);
		return 0; } 

	// if active, badframes are sent to the callback with buffersize = 0
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_CALLBACK_BR_FRAMES,1);
	if(result!=0){
		printf("Error: REG_CALLBACK_BR_FRAMES; Result = %d\n", result);
		return 0; } 

#ifdef DEFECT_CORRECTION
	result = NETUSBCAM_SetCamParameter(CamIndex, REG_DEFECT_COR,1);
	if(result!=0){
		printf("Error:  REG_DEFECT_COR; Result = %d\n", result);
		return 0; } 
#endif

	nRoiID = ROI_ID_1_1;			// start initialisation from roi 1
	for(i=0;i<nRoiAnz;i++)			
	{
		SetRoiID(nRoiID);		// set the roi active for initialisation
		context = (long*) nRoiID;	// use roi id as callback context
						// set callback for each roi 
		result = NETUSBCAM_SetCallback(CamIndex,CALLBACK_RGB,&GetImage,context);
		if(result!=0){
			printf("Error: SetCallback; Result = %d\n", result);
			return 0; } 

		SetExposure(nRoiID,fExp[nRoiID-1]); // it is possible to set exposure,
		SetGain(nRoiID,nGain[nRoiID-1]); // gain for each roi individually
 
		// set resolution for each roi (start from first,no gaps allowed)
		printf("Resolution: ROI_%d  XRes = %d, YRes = %d\n",(int)nRoiID,nXRes[nRoiID-1],nYRes[nRoiID-1]);

		result = NETUSBCAM_SetResolution(CamIndex,nXRes[nRoiID-1],nYRes[nRoiID-1],nXPos[nRoiID-1],nYPos[nRoiID-1]);
		if(result!=0){			
			printf("Error: SetResolution ROI_%d; Result = %d\n",(int)nRoiID,result);
			return 0; }

#ifdef ROI_CYCLE
		result = NETUSBCAM_SetCamParameter(CamIndex,REG_ROI_CYCLE,nCycleCnt[nRoiID-1]);
		if(result!=0){
			printf("Error: REG_ROI_CYCLE; Result = %d\n", result);
			return 0; } 
#endif
		nRoiID++; 
	}
 
#ifdef ROI_CYCLE		// save number of all rois
	for(i=0;i<nRoiAnz;i++)
		nRoisExpected += (nCycleCnt[i]+1);
#else 
	nRoisExpected = nRoiAnz;	
#endif

#ifdef TRIGGERMODE
	result = NETUSBCAM_SetTrigger(CamIndex,TRIG_SW_START);
	if(result!=0){
		printf("Error: SetTrigger; Result = %d\n", result);
		return 0; }
#endif
	result = NETUSBCAM_Start(CamIndex); // start streaming of camera
	if(result!=0){
		printf("Error: Start; Result = %d\n", result);
		return 0; }


	while(!bExit)
	{	  
		bImageSwTrig = false;   // callback flag
#ifdef TRIGGERMODE
		result = NETUSBCAM_SetTrigger(CamIndex, TRIG_SW_DO);
		if(result!=0){
			printf("Error: TRIG_SW_DO; Result = %d\n", result);
			return 0; }
		if (result == 0){
			int nCnt = 0;
			while(bImageSwTrig == false){
				usleep(10*1000); // 10 msec
				nCnt++;
				// 100 msec timeout (should be bigger than exposure time)
				if(nCnt >TRIGGER_TIMEOUT){
					printf("Timeout waiting for callback flag!\n"); 
					bExit=true; 
					break;
				}
			}
		}  
	
		if(nAllRoisCnt==1)	// all rois arrived one time, 
			bExit=true; 	// exit loop	
#else
		pause();		// wait for Ctrl+C
#endif			
		 
	}

	result = NETUSBCAM_Stop(CamIndex);	// stop streaming of camera
	if(result!=0){
		printf("Error: Stop; Result = %d\n", result);
		return 0; }
	
	nRoiID = nRoiAnz;
	for(i=0;i<nRoiAnz;i++)			// unset rois (start from last,no gaps allowed)
	{
		SetRoiID(nRoiID);
		// both functions are allowed to unset the roi
		//result = NETUSBCAM_SetResolution(CamIndex,0,0,0,0);
		result = NETUSBCAM_UnsetResolution(CamIndex);
		if(result!=0){
			printf("Error: UnsetResolution ROI_%d; Result = %d\n",(int)nRoiID,result);
			return 0; }	
		nRoiID--;
	}

	result = NETUSBCAM_Close(CamIndex);	// close camera
	if(result!=0){
		printf("Error: Close; Result = %d\n", result);
		return 0; }

	return 0;
}
//---------------------------------------------------------------------------
// save data from callback
void SaveRaw(unsigned char *buffer, unsigned int buffersize, const char* cName)
{
   FILE *outfile = fopen( cName, "wb");
   if ( !outfile ){
		printf("Error fopen\n");
		return; }	
   fwrite (buffer,1,buffersize,outfile );
   fclose( outfile );	
}
//---------------------------------------------------------------------------
//
void SignalHandler(int i)
{
   signal(SIGINT, SignalHandler);
   bExit=true;
}
//---------------------------------------------------------------------------
//
void CheckResolutionDelta(int nRes)
{
	int nTeiler = nRes % ROI_DELTA;		
	if(nTeiler!=0){		
		printf("Warning: resolution (%d) not dividable by ROI_DELTA \n",nRes);	
		printf("Warning: NETUSBCAM_SetResolution() will return error\n");	
	}
}
//---------------------------------------------------------------------------
//
void SetRoiID(int nRoiID)
{
#ifndef NO_4133_CAM
	int result = 0;	
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_ROI_ID,nRoiID);
	if(result!=0)
		printf("Error: SetRoiID; Result = %d\n", result);
#endif
}
//---------------------------------------------------------------------------
//
void SetExposure(int nRoiID,float nVal)
{
	float nRetVal;
	int result = 0;	

	SetRoiID(nRoiID);

	result = NETUSBCAM_SetExposure(CamIndex,nVal);
	if(result!=0)
		printf("Error: EXPOSURE; Result = %d\n", result);
}
//---------------------------------------------------------------------------
//
void SetGain(int nRoiID,int nVal)
{
	unsigned long nRetVal;
	int result = 0;	

	SetRoiID(nRoiID);

	result = NETUSBCAM_SetCamParameter(CamIndex,REG_GAIN,nVal);
	if(result!=0)
		printf("Error: GAIN; Result = %d\n", result);
}


/***********************************EOF****************************************/

